/*
**	IterateDirectory: File Manager directory iterator routines.
**
**	by Jim Luther
**
**	File:		IterateDirectory.c
**
**	Copyright  1995-1998 Jim Luther and Apple Computer, Inc.
**	All rights reserved.
**
**	You may incorporate this sample code into your applications without
**	restriction, though the sample code has been provided "AS IS" and the
**	responsibility for its operation is 100% yours.
**
**	IterateDirectory is designed to drop into the MoreFiles sample code
**	library I wrote while in Apple Developer Technical Support
*/

#include <Types.h>
#include <Errors.h>
#include <Files.h>
#include <stdio.h>
#include <string.h>


#include "osdep.h"
#include "macstuff.h"
#include "helpers.h"
#include "recurse.h"
#include "macglob.h"
#include "pathname.h"


int procname(char *n);  
int MatchWild( char *pStr,char *pPat );
extern int verbose;        /* 1=report oddities in zip file structure */


/*
**	Type definitions
*/

/* The RecurseGlobals structure is used to minimize the amount of
** stack space used when recursively calling RecurseDirectoryLevel
** and to hold global information that might be needed at any time.
*/
#if PRAGMA_ALIGN_SUPPORTED
#pragma options align=mac68k
#endif
struct RecurseGlobals
{
	short					vRefNum;
	CInfoPBRec				cPB;			/* the parameter block used for PBGetCatInfo calls */
	unsigned char			itemName[NAME_MAX];		/* the name of the current item */
	char 					FullPath[NAME_MAX];
	short 					FullPathLen;
	OSErr					result;			/* temporary holder of results - saves 2 bytes of stack each level */
	Boolean					quitFlag;		/* set to true if filter wants to kill interation */
	unsigned short			maxLevels;		/* Maximum levels to iterate through */
	unsigned short			currentLevel;	/* The current level IterateLevel is on */
};
#if PRAGMA_ALIGN_SUPPORTED
#pragma options align=reset
#endif

typedef struct RecurseGlobals RecurseGlobals;
typedef RecurseGlobals *RecurseGlobalsPtr;

extern MacZipGlobals	MacZip;
extern const char ResourceMark[255]; /* var is initialized in file pathname.c */
static RecurseGlobals theGlobals;

static unsigned long	DirLevels = 0;

/*****************************************************************************/
/*	Static Prototype */

static	void	RecurseDirectoryLevel(long DirID, RecurseGlobals *Globals);

/*****************************************************************************/

/*
**	Functions
*/

static	void	RecurseDirectoryLevel(long DirID, RecurseGlobals *Globals)
{
OSErr		rc;
Boolean		isInvisible, isAlias, nameLocked;
Boolean		IncludeItem;
Boolean		hasDataFork;
Boolean		hasResourceFork;
char		buffer[NAME_MAX];

	if ( (Globals->maxLevels == 0) ||					/* if maxLevels is zero, we aren't checking levels */
		 (Globals->currentLevel < Globals->maxLevels) )	/* if currentLevel < maxLevels, look at this level */
	{
		short index = 1;
		
		++Globals->currentLevel;	/* go to next level */
		if (DirLevels < Globals->currentLevel) DirLevels = Globals->currentLevel;
		
		do
		{	/* Isn't C great... What I'd give for a "WITH theGlobals DO" about now... */
		
			/* Get next source item at the current directory level */
			
			Globals->cPB.dirInfo.ioFDirIndex = index;
			Globals->cPB.dirInfo.ioDrDirID = DirID;
			Globals->result = PBGetCatInfoSync((CInfoPBPtr)&Globals->cPB);		
			
	
			if ( Globals->result == noErr ) 
			{	
				isInvisible = !((Globals->cPB.hFileInfo.ioFlFndrInfo.fdFlags & fInvisible) == 0 );	
				isAlias		= !((Globals->cPB.hFileInfo.ioFlFndrInfo.fdFlags & 0x8000) == 0);
				nameLocked  = !((Globals->cPB.hFileInfo.ioFlFndrInfo.fdFlags & 0x1000) == 0);
				
				IncludeItem = !isInvisible;
				if ((MacZip.IncludeInvisible) && (isInvisible)) IncludeItem = true;
				
				/* Is it a File? */
				if ( (Globals->cPB.hFileInfo.ioFlAttrib & ioDirMask) == 0 )
				{
				PToCCpy(Globals->itemName,MacZip.FileName);
				MacZip.isDirectory = false;
				
				hasDataFork     = (Globals->cPB.hFileInfo.ioFlLgLen != 0);
				hasResourceFork = (Globals->cPB.hFileInfo.ioFlRLgLen != 0);
			    
				/* include also files with zero recource- & data-fork */
				if ((hasDataFork == 0) & (hasResourceFork == 0)) hasDataFork = true;
				
				GetFullPathFromID(Globals->FullPath,Globals->vRefNum, DirID, Globals->itemName, &rc);
				printerr("GetFullPathFromID:",rc,rc,__LINE__,__FILE__,MacZip.FileName);
				
				/* don't include the zipfile itself */
	            if (!nameLocked && !isAlias && IncludeItem  && (!IsZipFile(Globals->FullPath))   ) 
	            	{
	            	
	            	if (MatchWild( MacZip.FileName,MacZip.Pattern ) == true)
	            		{
	            		/* some file statistics */
	            	  	MacZip.FoundFiles++;    

	            		if (hasDataFork == true) 
	            			{ 
	            			MacZip.BytesOfData = Globals->cPB.hFileInfo.ioFlLgLen + MacZip.BytesOfData;
	            			MacZip.CurrentFork = DataFork;  
	            			
	            			if (MacZip.DataForkOnly == true)  
	            				{
	            				procname(Globals->FullPath);
	            				}
	            				else 
	            				{
	            				procname(Real2RfDfFilen(buffer,Globals->FullPath, DataFork));
	            				}
	            			}
	            		 
	            		if (hasResourceFork == true)
	            			{ 
	            			MacZip.BytesOfData = Globals->cPB.hFileInfo.ioFlRLgLen + MacZip.BytesOfData;
	            			MacZip.CurrentFork = ResourceFork; 
	            			procname(Real2RfDfFilen(buffer, Globals->FullPath, ResourceFork));
	            			}
	            		} /* if (MatchWild( MacZip.FileName,MacZip.Pattern ) == true) */
	            	}  /* if (!IsZipFile(Globals->FullPath)) */
				} /* Is it a File? */

				/* Is it a directory? */
				if ( (Globals->cPB.hFileInfo.ioFlAttrib & ioDirMask) != 0 )
				{
                if (verbose) printf(".");
                MacZip.isDirectory = true;
                
  				GetFullPathFromID(Globals->FullPath,Globals->vRefNum, DirID, Globals->itemName, &rc);
						
				if  ((MacZip.StoreFoldersAlso) && (IncludeItem))
					{
					procname(Globals->FullPath); 
					MacZip.FoundDirectories++;
					}
				else MacZip.FoundDirectories++;
					
				     /* We have a directory */
					if ( !Globals->quitFlag )
					{
						/* Dive again if the IterateFilterProc didn't say "quit" */
						RecurseDirectoryLevel(Globals->cPB.dirInfo.ioDrDirID, Globals);
					}
				} /* Is it a directory? */
			} /* if ( Globals->result == noErr ) */
			
			++index; /* prepare to get next item */
		} while ( (Globals->result == noErr) && (!Globals->quitFlag) ); /* time to fall back a level? */
		
		if ( (Globals->result == fnfErr) ||	/* fnfErr is OK - it only means we hit the end of this level */
			 (Globals->result == afpAccessDenied) ) /* afpAccessDenied is OK, too - it only means we cannot see inside a directory */
		{
			Globals->result = noErr;
		}
			
		--Globals->currentLevel;	/* return to previous level as we leave */
	}
}



/*****************************************************************************/

pascal	OSErr	RecurseDirectory(short vRefNum,
								 long thedirID,
								 ConstStr255Param name,
								 unsigned short maxLevels)
{
	OSErr			result;
	short			theVRefNum;
	Boolean			isDirectory;
	long			DirID;


		/* Get the real directory ID and make sure it is a directory */
		result = GetDirectoryID(vRefNum, thedirID, name, &DirID, &isDirectory);
		if ( result == noErr )
		{
			if ( isDirectory == true )
			{
				/* Get the real vRefNum */
				result = DetermineVRefNum(name, vRefNum, &theVRefNum);
				if ( result == noErr )
				{
					/* Set up the globals we need to access from the recursive routine. */
					theGlobals.cPB.hFileInfo.ioNamePtr = theGlobals.itemName;
					theGlobals.cPB.hFileInfo.ioVRefNum = theVRefNum;
					theGlobals.itemName[0] = 0;
					theGlobals.result = noErr;
					theGlobals.quitFlag = false;
					theGlobals.maxLevels = maxLevels;
					theGlobals.currentLevel = 0;	/* start at level 0 */
				
					/* Here we go into recursion land... */ 
					RecurseDirectoryLevel(DirID, &theGlobals);
					
					result = theGlobals.result;	/* set the result */
				}
			}
			else
			{
				result = dirNFErr;	/* a file was passed instead of a directory */
			}
		}
	
	return ( result );
}



/*****************************************************************************/

pascal	OSErr	FSpRecurseDirectory(const FSSpec *spec,
									unsigned short maxLevels)
{
OSErr rc;

	theGlobals.vRefNum = spec->vRefNum;
	
	if ((noisy) && (MacZip.DataForkOnly)) 
		printf("\n Warning: Datafork only \n");
			
	if (noisy) printf("\n Collecting Filenames ...");
	rc = RecurseDirectory(spec->vRefNum, spec->parID, spec->name,maxLevels);
	printerr("RecurseDirectory:",rc,rc,__LINE__,__FILE__,"");
	
	if (noisy) printf("\n... done \n\n %6d matched files found  \n",MacZip.FoundFiles);
	if (noisy) printf(" %6d folders found in %d Levels \n",MacZip.FoundDirectories,DirLevels);
	
	if (MacZip.BytesOfData > (1024*1024)) 
		if (noisy) printf(" %4.3f MBytes unzipped size\n\n",(float) MacZip.BytesOfData/(1024*1024));
	else
		if (noisy) printf(" %4.3f KBytes unzipped size\n\n",(float) MacZip.BytesOfData/1024);
	
	return rc;
}





