/*

 Copyright (C) Dirk Haase
 Permission is granted to any individual or institution to use, copy, or
 redistribute this software so long as all of the original files are included,
 that it is not sold for profit, and that this copyright notice is retained.

*/

#ifndef __OSDEP_H
#define __OSDEP_H 1


#include <setjmp.h>
#include <types.h>
#include <unistd.h>
#include <signal.h>
#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <console.h>
#include <unix.h> 

#include <Errors.h>
#include <Files.h>

#include "unixlike.h"
#include "macglob.h"


#define NO_MKTEMP           1 
#define HAVE_DIRENT_H       1
#define PASSWD_FROM_STDIN   1

#define USE_ZIPMAIN			1
#define USE_ZIPSPLITMAIN    1
#define USE_ZIPNOTEMAIN     1

int 	zipsplitmain(int argc,char **argvalue);
int 	zipnotemain(int argc,char **argvalue);

#define NOCPYRT 

#define USE_EF_UT_TIME      1   /* Enable use of "UT" extra field time info */


#define EXTRAFLD_MAX    0xFFFF

#define EXDEV  18

#define MATCH shmatch


#define OLDROUTINENAMES     0			/* use new function names only */
#define OLDROUTINELOCATIONS 0			/* use new headerlocations only */
#define SystemSevenOrLater 	1	        /* Runs only on System 7.0 or later	*/


/* file operations use "b" for binary */
#define FOPR  "rb"
#define FOPM  "r+b"
#define FOPW  "wb"

/*
#define DEBUG
*/

#define ADD     1
#define UPDATE  2
#define FRESHEN 3

/* These functions are defined as a macro instead of a function.
so we have to undefine them for replacing (see printf.c)  */
#undef getc
#undef getchar
#undef putchar 



void setfiletype(char *file, unsigned long Creator, unsigned long Type);
int deletedir(char *);

void version_local ();
char *GetZipVersionsInfo(void);
char *GetZipVersionLocal(void);
char *GetZipCopyright(void);

void InitAllVars(void);

char *in2ex(char *n);
char *ex2in(char *x, int isdir, int *pdosflag);
void PrintFileInfo(void);





/* Many thinks are different for mac-users, so we need spezial mac functions :-)  */
int MacStat (char *path, struct stat *buf);
#define stat(path, bufPtr) MacStat(path, bufPtr)
#define lstat(path, bufPtr) MacStat(path, bufPtr)

int fprintf(FILE *file, const char *format, ...);
int printf(const char *format, ...);
void perror(const char *parm1);

int MacOpen(char *path,int oflag);
FILE *MacFopen(char *path,const char *mode);
#define fopen(path, mode)	MacFopen(path, mode)
#define open(path, oflag) 	MacOpen(path, oflag)  


char *GetComment(char *filename);
int readlink(char *path, char *buf, int size);

void PrintStatProgress(char *msg);

/*
The following macro transforms this statement e.g:
if ((r = PROCNAME(argv[i])) != ZE_OK)

into this statement:
if ((r = (action == 1 || action == 2 ? wild(argv[i]) : procname(argv[i]))) != 0)

Just to make it clear :-)
The zip.c calls now (if this macro is active) also the function wild(char *)

*/

#define PROCNAME(n) (action == ADD || action == UPDATE ? wild(n) : procname(n))

extern int noisy;   /* flag for regular output */
extern int verbose;

#endif   /* _osdep_h_ */
